package com.youkeda.ykd_audio_recorder;


import android.Manifest;
import android.content.pm.PackageManager;
import android.media.MediaRecorder;
import android.os.Build;
import android.os.Environment;
import android.util.Log;

import androidx.core.app.ActivityCompat;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;


/**
 * YkdAudioRecorderPlugin
 */
public class YkdAudioRecorderPlugin implements MethodCallHandler {
    //录音API
    private MediaRecorder mMediaRecorder;

    //录音文件保存位置
    private static String mFilePath = Environment.getExternalStorageDirectory().getAbsolutePath() + "/audio/";

    private String mSavePath;

    private long startTime;

    private static PluginRegistry.Registrar reg;

    /**
     * Plugin registration.
     */
    public static void registerWith(PluginRegistry.Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), "ykd_audio_recorder");
        channel.setMethodCallHandler(new YkdAudioRecorderPlugin());
        reg = registrar;
//    if(com.youkeda.ykd_audio_recorder.CustomPhoneStateListener.status){
//      channel.invokeMethod("interrupt","END");
//    }else {
//      channel.invokeMethod("interrupt","BEGIN");
//    }
    }


    @Override
    public void onMethodCall(MethodCall call, Result result) {
        if (call.method.equals("start")) {
            this.start(call, result);
        } else if (call.method.equals("stop")) {
            this.stop(result);
        } else if (call.method.equals("pause")) {
            this.pause();
        } else if (call.method.equals("getVolumn")) {
            result.success(this.getVolumn());
        } else if (call.method.equals("resume")) {
            this.resume();
        }
    }

    private void start(MethodCall call, Result result) {
        Map<String, Object> startResult = new HashMap<>();
        Map params = (HashMap) call.arguments;

        //处理路径问题，如果没有传递path，则创建一个文件
        String path;
        if (params.containsKey("path")) {
            path = (String) params.get("path");
        } else {
            path = mFilePath + System.currentTimeMillis() + ".m4a";
        }

        mSavePath = path;
        //创建文件
        boolean createResult = this.createFile(path);
        if (!createResult) {
            startResult.put("success", false);
            startResult.put("error", "文件创建失败");
            result.success(startResult);
            return;
        }

        boolean isRecordSuccess = this.startRecord(path);
        startResult.put("success", isRecordSuccess);
        startResult.put("error", "录音创建失败");
        result.success(startResult);
    }

    private void stop(Result result) {
        this.stopRecord();
        Map<String, Object> stopResult = new HashMap<>();
        stopResult.put("path", this.mSavePath);
        stopResult.put("duration", System.currentTimeMillis() - this.startTime);
        result.success(stopResult);
    }

    private boolean createFile(String filePath) {
        Log.d("VVVVVV", filePath);
        File file = new File(filePath);
        if (file.exists()) {
            return true;
        }

        //判断父目录是否存在
        if (!file.getParentFile().exists()) {
            //父目录不存在 创建父目录
            if (!file.getParentFile().mkdirs()) {
                return false;
            }
        }

        //创建目标文件
        try {
            if (file.createNewFile()) {//创建文件成功
                return true;
            }
        } catch (IOException e) {
            return false;
        }

        return false;
    }

    /**
     * @description 录音操作
     * @author ldm
     */
    private boolean startRecord(String path) {
        if (ActivityCompat.checkSelfPermission(reg.activity(), Manifest.permission.RECORD_AUDIO) != PackageManager.PERMISSION_GRANTED
                || ActivityCompat.checkSelfPermission(reg.activity(), Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED
        ) {
            ActivityCompat.requestPermissions(reg.activity(), new String[]{
                    Manifest.permission.RECORD_AUDIO,
                    Manifest.permission.WRITE_EXTERNAL_STORAGE,
            }, 0);
            Log.d("VOICE", "获取权限失败");
            return false;
        }
        //创建MediaRecorder对象
        mMediaRecorder = new MediaRecorder();
        try {
            //配置mMediaRecorder相应参数
            //从麦克风采集声音数据
            mMediaRecorder.setAudioSource(MediaRecorder.AudioSource.MIC);
            //设置保存文件格式为MP4
            mMediaRecorder.setOutputFormat(MediaRecorder.OutputFormat.MPEG_4);
            //设置采样频率,44100是所有安卓设备都支持的频率,频率越高，音质越好，当然文件越大
            mMediaRecorder.setAudioSamplingRate(44100);
            //设置声音数据编码格式,音频通用格式是AAC
            mMediaRecorder.setAudioEncoder(MediaRecorder.AudioEncoder.AAC);
            //设置编码频率
            mMediaRecorder.setAudioEncodingBitRate(96000);
            //设置录音保存的文件
            mMediaRecorder.setOutputFile(path);
            //开始录音
            mMediaRecorder.prepare();
            mMediaRecorder.start();
            //记录开始录音时间
            startTime = System.currentTimeMillis();
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            Log.d("voice", e.toString());
            return false;
        }
    }


    /**
     * @description 结束录音操作
     * @author ldm
     */
    private void stopRecord() {
        mMediaRecorder.stop();
        if (null != mMediaRecorder) {
            mMediaRecorder.release();
            mMediaRecorder = null;
        }
    }

    /**
     * @description 暂停录音
     * @author ldm
     */
    public void pause() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            mMediaRecorder.pause();
        }
    }

    /**
     * @description 恢复录音
     * @author ldm
     */

    public void resume() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            mMediaRecorder.resume();
        }
    }

    private int BASE = 1;

    public double getVolumn() {
        double db = 0;
        if (mMediaRecorder != null) {
            double ratio = (double) mMediaRecorder.getMaxAmplitude() / BASE;
            if (ratio > 1) {
                db = 17 * Math.log10(ratio);
            }
        }
        return db / 100;
    }
}
